/****************************************************************************************/
/* Basis-Unterprogramme zur Ansteuerung des Grafik-LCD (mit SED1520-Controller)         */
/* in C (CC5X)																			*/
/*																						*/
/* Entwickler: Buchgeher Stefan															*/
/* Entwicklungsbeginn der Software: 5. Maerz 2006										*/
/* Funktionsfaehig seit: 2. April 2006 					 							    */
/* Letzte Bearbeitung: 3. April 2006													*/
/****************************************************************************************/

/****************** Projekt-Header einbinden ********************************************/
#include "PROJEKT.H"


/****************** externe Register ****************************************************/
unsigned char hlp_zeilen_zaehler;
unsigned char hlp_spalten_zaehler;


/****************** Basis-Unterprogramme zur Grafik-LCD-Ansteuerung *********************/

/****************************************************************************************/
/* GLCD_INIT                                                                            */
/*                                                                                      */
/* Aufgabe:                                                                             */
/*	 Grafik-LC-Display initialisieren						     						*/
/*																						*/
/* Uebergabeparameter:                                                                  */
/*	 keine																				*/
/*																						*/
/* Rueckgabeparameter:                                                                  */
/*	 keiner																				*/
/*                                                                                      */
/* Vorgehensweise:                                                                      */
/*	 Uebergibt nacheinander die Befehle zur Initialisierung an das Grafik-LC-Display. 	*/
/****************************************************************************************/
void GLCD_INIT(void)
{
  	GLCD_COMMAND(175);					// Display on
	GLCD_COMMAND(164); 					// static drive off
	GLCD_COMMAND(169);					// duty cyle: 1/32
	GLCD_COMMAND(238);					// read modify write off
	GLCD_COMMAND(192);					// line 0
	GLCD_COMMAND(184);					// 1. Zeile
	GLCD_COMMAND(0);					// 1. Spalte ("Column Address" auf 0 setzen)
}


/****************************************************************************************/
/* GLCD_COMMAND                                                                         */
/*                                                                                      */
/* Aufgabe:                                                                             */
/*	 Umsetzung des Zeitdiagramms zum Senden eines Befehls an das Grafik-LC-Display    	*/
/*                                                                                      */
/*                       _____                       ________                           */
/*             A0        _____X_____________________X________                           */
/*                       _____                       ________                           */
/*             nCS1,nCS2 _____X_____________________X________                           */
/*                       ____________________________________                           */
/*             nRD					                                     				*/
/*                       ________                 ___________                           */
/*             nWR               \_______________/                                      */
/*                       ___________ ________________ _______                           */
/*             Data      ___________X________________X_______                           */
/*																						*/
/* Uebergabeparameter:                                                                  */
/*	 command																			*/
/*																						*/
/* Rueckgabeparameter:                                                                  */
/*	 keiner																				*/
/****************************************************************************************/
void GLCD_COMMAND(unsigned char command)
{
	GLCD_A0 = 0;
	GLCD_CS1 = 0;
	GLCD_CS2 = 0;	 

	GLCD_RD = 1;
	GLCD_WR = 0;

	GLCD_DATA = command;

	GLCD_WR = 1;

	GLCD_CS1 = 1;
	GLCD_CS2 = 1;
}


/****************************************************************************************/
/* GLCD_DATA1  									     									*/
/*                                                                                      */
/* Aufgabe:                                                                             */
/*	 Umsetzung des Zeitdiagramms zum Senden von Daten an die erste Haelfte des         	*/
/*	 Grafik-LC-Display                                                                	*/
/*                                                                                      */
/*                       _____ _____________________ ________                           */
/*             A0, nCS1  _____X                     X________                          	*/
/*                       _____                       ________                           */
/*             nCS2      _____X_____________________X________                           */
/*                       ____________________________________                           */
/*             nRD					                                     				*/
/*                       ________                 ___________                           */
/*             nWR               \_______________/                                      */
/*                       ___________ ________________ _______                           */
/*             Data      ___________X________________X_______                           */
/*																						*/
/* Uebergabeparameter:                                                                  */
/*	 data																				*/
/*																						*/
/* Rueckgabeparameter:                                                                  */
/*	 keiner																				*/
/****************************************************************************************/
void GLCD_DATA1(unsigned char data)
{
	GLCD_A0 = 1;
	GLCD_CS1 = 1;
	GLCD_CS2 = 0;	 

	GLCD_RD = 1;
	GLCD_WR = 0;

	GLCD_DATA = data;

	GLCD_WR = 1;

	GLCD_CS1 = 1;
	GLCD_CS2 = 1;
}


/****************************************************************************************/
/* GLCD_DATA2  									     									*/
/*                                                                                      */
/* Aufgabe:                                                                             */
/*	 Umsetzung des Zeitdiagramms zum Senden von Daten an die zweite Haelfte des         */
/*   Grafik-LC-Display                                                                	*/
/*                                                                                      */
/*                       _____ _____________________ ________                           */
/*             A0, nCS2  _____X                     X________                          	*/
/*                       _____                       ________                           */
/*             nCS1      _____X_____________________X________                           */
/*                       ____________________________________                           */
/*             nRD					                                     				*/
/*                       ________                 ___________                           */
/*             nWR               \_______________/                                      */
/*                       ___________ ________________ _______                           */
/*             Data      ___________X________________X_______                           */
/*																						*/
/* Uebergabeparameter:                                                                  */
/*	 data																				*/
/*																						*/
/* Rueckgabeparameter:                                                                  */
/*	 keiner																				*/
/****************************************************************************************/
void GLCD_DATA2(unsigned char data)
{
	GLCD_A0 = 1;
	GLCD_CS1 = 0;
	GLCD_CS2 = 1;	 

	GLCD_RD = 1;
	GLCD_WR = 0;

	GLCD_DATA = data;

	GLCD_WR = 1;

	GLCD_CS1 = 1;
	GLCD_CS2 = 1;
}  
 

/****************************************************************************************/
/* GLCD_DATA_ZEICHEN  									     							*/
/*                                                                                      */
/* Aufgabe:                                                                             */
/*	 Je nach aktueller Position des Cursors den Wert (data) an die erste oder zweite    */
/*   Displayhaelfte uebergeben.                                                         */
/*																						*/
/* Uebergabeparameter:                                                                  */
/*	 data																				*/
/*																						*/
/* Rueckgabeparameter:                                                                  */
/*	 keiner																				*/
/*																						*/
/* Vorgehensweise:																		*/
/*   + Befindet sich der Cursor an der 62. Spalte (hlp_spalten_zaehler = 61), die so-   */
/*     genannte "Column Address" auf 0 setzen (gemaess Datenblatt). Dies bewirkt, dass 	*/
/*	   der Display-interne Speicherzaehler wieder bei 0 beginnt. (siehe Datenblatt)		*/
/*   + Befindet sich der Cursor an eine Spalte groesser als 61 (also zweite Display- 	*/
/*	   haelfte) den Uebergabewert (data) mit dem Unterprogramm GLCD_DATA2 an die 		*/
/*	   zweite Displayhaelfte uebergeben. Andernfalls den Uebergabewert (data) mit dem 	*/
/*	   Unterprogramm GLCD_DATA1 an die erste Displayhaelfte uebergeben.					*/
/*																						*/
/* Anmerkung:                                                                  			*/
/*	 Das (externe) Hilfsregister hlp_spalten_zaehler gibt an, an welcher Spalte sich    */
/*   der Cursor befindet.																*/	
/****************************************************************************************/
void GLCD_DATA_ZEICHEN(unsigned char data)
{
	if (hlp_spalten_zaehler == 61)		// Wenn Cursorspalte = 61 ...
	{
		GLCD_COMMAND(0);				//   ... "Column Address" auf 0 setzen
	}
	if (hlp_spalten_zaehler > 60)		// Cursorspalte > 60
	{
  		GLCD_DATA2(data);				// ja: Uebergabewert (data) mit dem an 
	}									//   die zweite Displayhaelfte uebergeben.
	else
	{
  		GLCD_DATA1(data);			 	// nein: Uebergabewert (data) mit dem an
	}									//   die erste Displayhaelfte uebergeben.
	hlp_spalten_zaehler++;
}


/****************************************************************************************/
/* GLCD_CLEAR										     								*/
/*                                                                                      */
/* Aufgabe:                                                                             */
/*	 Loeschen des Grafik-LC-Displays                                                  	*/
/*                                                                                      */
/* Uebergabeparameter:                                                                  */
/*   keine																				*/
/*																						*/
/* Rueckgabeparameter:                                                                  */
/*   keiner																				*/
/*																						*/
/* Vorgehensweise:                                                                      */
/*   Beginnend bei der ersten Zeile und ersten Spalte mit Hilfe zweier geschachtelten	*/
/*   Schleifen Nullen zum Grafik-LC-Display (in beide Display-Haelften) schreiben. Dies */
/*	 bewirkt, dass am Display nichts angezeigt wird bzw. das Display wird somit ge-		*/
/*	 loescht.																			*/
/****************************************************************************************/
void GLCD_CLEAR(void)
{
	unsigned char Zeile, Spalte;


	for(Zeile = 0; Zeile < 4; Zeile++)
	{
		GLCD_COMMAND(184 + Zeile);				// Befehl: naechste Zeile
		
		GLCD_COMMAND(0);						// Befehl: 1.Spalte der Zeile ("Column  
												//  Address" auf 0 setzen)
		for(Spalte = 0; Spalte < 61; Spalte++)
		{
			GLCD_DATA1(0);
			GLCD_DATA2(0);
		}	
	}
}


/****************************************************************************************/
/* GLCD_SETCURSOR_ZEILE										     						*/
/*                                                                                      */
/* Aufgabe:                                                                             */
/*	 Den Cursor an die uebergebene Zeile setzen									      	*/
/*																						*/
/* Uebergabeparameter:                                                                  */
/*   zeile:  0 - 3																		*/
/*																						*/
/* Rueckgabeparameter:                                                                  */
/*   keiner																				*/
/*                                                                                      */
/* Vorgehensweise:                                                                      */
/*	 Uebergabeparameter fuer die Zeile im globalen Register hlp_zeilen_zaehler sichern  */
/*   und an das Grafik-LC-Display uebergeben.										    */
/****************************************************************************************/
void GLCD_SETCURSOR_ZEILE(unsigned char zeile)
{
	hlp_zeilen_zaehler = zeile;
	GLCD_COMMAND(184 + zeile);
}


/****************************************************************************************/
/* GLCD_SETCURSOR_SPALTE										     					*/
/*                                                                                      */
/* Aufgabe:                                                                             */
/*	 Den Cursor an die uebergebene Spalte setzen                 						*/
/*																						*/
/* Uebergabeparameter:                                                                  */
/*   spalte: 0 - 121																	*/
/*																						*/
/* Rueckgabeparameter:                                                                  */
/*   keiner																				*/
/*                                                                                      */
/* Vorgehensweise:                                                                      */
/*   Uebergabeparameter fr die Spalte im globalen Register hlp_spalten_zaehler sichern */
/*	 und an das Grafik-LC-Display uebergeben. Dabei ist zu beachten, dass das Grafik-LC-*/
/*   Display in zwei Bereiche aufgeteilt ist, wobei beide Bereiche mit 0 beginnen.      */
/*   Spalten groesser als 61 befinden sich im zweiten Bereich. Da diese wieder ab 0     */
/*   beginnen muessen bei den Spalten die groesser als 61 sind 61 abgezogen werden.		*/
/****************************************************************************************/
void GLCD_SETCURSOR_SPALTE(unsigned char spalte)
{
	hlp_spalten_zaehler = spalte;
	if (spalte < 61)
	{
  		GLCD_COMMAND(spalte);
	}	
	else
	{
  		GLCD_COMMAND(spalte - 61);
	}	
}



 