/****************************************************************************************/
/* Unterprogramme zur Ansteuerung des Grafik-LCD (mit SED1520-Controller) in C (CC5X)	*/
/* (Font 8x14)																			*/
/*																						*/
/* Entwickler: Buchgeher Stefan															*/
/* Entwicklungsbeginn der Software: 8. April 2006										*/
/* Funktionsfaehig seit: 8. April 2006 					 							    */
/* Letzte Bearbeitung: 10. April 2006													*/
/****************************************************************************************/

/****************** Projekt-Header einbinden ********************************************/
#include "PROJEKT.H"
 

/****************** Unterprogramme zur Grafik-LCD-Ansteuerung ***************************/

/****************************************************************************************/
/* GLCD_ZEICHEN_8x14  									     							*/
/*                                                                                      */
/* Aufgabe:                                                                             */
/*	 Ein Zeichen (Charakter im Format 8x14) am Grafik-LC-Display ausgeben		    	*/
/*                                                                                      */
/* Uebergabeparameter:                                                                  */
/* 	 zeichen			                                                                */
/*                                                                                      */
/* Rueckgabeparameter:                                                                  */
/*	 keiner																				*/
/*                                                                                      */
/* Vorgehensweise:                                                                      */
/*   + die externen Hilfsregister (hlp_zeilen_zaehler und hlp_spalten_zaehler) sichern, */
/*     da sie hier benoetigt, aber gleichzeitig hier auch veraendert werden)			*/
/*	 + Nacheinander die Spalten fuer die erste Zeile des auszugebenden Zeichens an das  */
/*     Grafik-LC-Display mit dem Unterprogramm GLCD_DATA_ZEICHEN uebergeben. Die Daten  */
/*	   fuer die Spalten sind in Tabellen abgelegt, wobei fuer jede Spalte des Zeichens  */
/*	   eine eigene Tabelle vorhanden ist. Diese Tabellen muessen an anderer Stelle 		*/
/*	   hinzugefuegt werden! Die letzte Spalte muss eine 0 enthalten, also eine "Leer-	*/
/*	   spalte".																			*/
/*	 + Den Cursor an die naechste Zeile (tmp_zeile + 1) und die erste Spalte des 		*/
/*     Zeichens (tmp_spalte) setzen.													*/
/*   + In gleicher weise wie vorher die Spalten fuer die zweite Zeile des auszugebenden */
/*	   Zeichens an das Grafik-LC-Display mit dem Unterprogramm GLCD_DATA_ZEICHEN 		*/
/*	   uebergeben.																		*/
/*   + Den Cursor an die erste Zeile des Zeichens (tmp_zeile) und an die Spalte vom 	*/
/*	   Ende des Zeichen	(tmp_spalte + 12) setzen										*/
/****************************************************************************************/
void GLCD_ZEICHEN_8x14(unsigned char zeichen)
{
	char tmp_zeile = hlp_zeilen_zaehler;
	char tmp_spalte = hlp_spalten_zaehler;
  
	
	// 1.Zeile
	// 1. Spalte des Zeichens 
	GLCD_DATA_ZEICHEN(TabZeichensatz8x14_z1sp1[zeichen]);
	// 2. Spalte des Zeichens 
	GLCD_DATA_ZEICHEN(TabZeichensatz8x14_z1sp2[zeichen]);
	// 3. Spalte des Zeichens 
	GLCD_DATA_ZEICHEN(TabZeichensatz8x14_z1sp3[zeichen]);
	// 4. Spalte des Zeichens 
	GLCD_DATA_ZEICHEN(TabZeichensatz8x14_z1sp4[zeichen]);
	// 5. Spalte des Zeichens 
	GLCD_DATA_ZEICHEN(TabZeichensatz8x14_z1sp5[zeichen]);
	// 6. Spalte des Zeichens 
	GLCD_DATA_ZEICHEN(TabZeichensatz8x14_z1sp6[zeichen]);
	// 7. Spalte des Zeichens 
	GLCD_DATA_ZEICHEN(TabZeichensatz8x14_z1sp7[zeichen]);
	// 8. Spalte des Zeichens 
	GLCD_DATA_ZEICHEN(TabZeichensatz8x14_z1sp8[zeichen]);
	// 9. Spalte des Zeichens 
	GLCD_DATA_ZEICHEN(0);


	// 2. Zeile und 1.Spalte des Zeichen
	GLCD_SETCURSOR_ZEILE(tmp_zeile + 1);
	GLCD_SETCURSOR_SPALTE(tmp_spalte);

	// 1. Spalte des Zeichens
	GLCD_DATA_ZEICHEN(TabZeichensatz8x14_z2sp1[zeichen]);
	// 2. Spalte des Zeichens 
	GLCD_DATA_ZEICHEN(TabZeichensatz8x14_z2sp2[zeichen]);
	// 3. Spalte des Zeichens 
	GLCD_DATA_ZEICHEN(TabZeichensatz8x14_z2sp3[zeichen]);
	// 4. Spalte des Zeichens 
	GLCD_DATA_ZEICHEN(TabZeichensatz8x14_z2sp4[zeichen]);
	// 5. Spalte des Zeichens 
	GLCD_DATA_ZEICHEN(TabZeichensatz8x14_z2sp5[zeichen]);
	// 6. Spalte des Zeichens 
	GLCD_DATA_ZEICHEN(TabZeichensatz8x14_z2sp6[zeichen]);
	// 7. Spalte des Zeichens 
	GLCD_DATA_ZEICHEN(TabZeichensatz8x14_z2sp7[zeichen]);
	// 8. Spalte des Zeichens 
	GLCD_DATA_ZEICHEN(TabZeichensatz8x14_z2sp8[zeichen]);
	// 9. Spalte des Zeichens 
	GLCD_DATA_ZEICHEN(0);

 
	// 1. Zeile und Spalte vom Ende des Zeichen	
	GLCD_SETCURSOR_ZEILE(tmp_zeile);
 }

