/****************************************************************************************/
/* Unterprogramme zur PID-Regelung in C (CC5X)											*/
/*																						*/
/* Entwickler: Buchgeher Stefan															*/
/* Entwicklungsbeginn der Software: 20. April 2006										*/
/* Funktionsfaehig seit:  					 							    */
/* Letzte Bearbeitung: 1. Mai 2006													*/
/****************************************************************************************/

/****************** Projekt-Header einbinden ********************************************/
#include "PROJEKT.H"


/****************** externe Register ****************************************************/
uns16 tmp_uebergabe_uns16;
int16 tmp_uebergabe_int16;

uns16 sollwert;
uns16 istwert;

int16 e_alt;				// vorhergehende Regelabweichung
int16 e_neu;				// neue Regelabweichung
int16 y_alt;				// vorhergehende Stellgroesse
int16 y_neu;				// neue Stellgroesse

uns16 Kp;
uns16 Ki;
uns16 Kd;

unsigned char PID_STATUS;	// Statusregister fuer PID-Regler
//bit pid_ein @ PID_STATUS.0;			// Globale aktivierung der PID-Regelung
//bit pid_p_aktiv @ PID_STATUS.5;		// Bit gesetzt: P-Anteil wird im PID-Algorithmus 
									//   beruecksichtigt
//bit pid_i_aktiv @ PID_STATUS.6;		// Bit gesetzt: I-Anteil wird im PID-Algorithmus 
									//   beruecksichtigt
//bit pid_d_aktiv @ PID_STATUS.7;		// Bit gesetzt: D-Anteil wird im PID-Algorithmus 
									//   beruecksichtigt



/****************** Unterprogramm zur Ist- und Sollwerteingabe **************************/

/****************************************************************************************/
/* ADC:																					*/
/*																						*/
/* Aufgabe:																				*/
/*	 Analog-Digital-Wandlung mit dem PIC-interne ADC fuer den ausgewaehlten ADC-Kanal   */
/*	 starten, auf das Wandlungsergebnis warten und dieses dem aufrufenden Programm 		*/
/*	 zurueckgeben.																		*/
/*																						*/
/* Uebergabeparameter:                                                                  */
/*	 kanal: 			                                                          		*/
/*																						*/
/* Rueckgabeparameter:                                                                  */
/*	 ergebnis:  16-Bit-Ergebnis der Analog-Digital-Wandlung								*/
/*                                                                                      */
/* Vorgehensweise: 																		*/
/*	 + ADC-Kanal auswaehlen (Bits <3:5> im Register ADCON0). Dazu muss die Kanal-Nummer */
/*     an diese Position geschoben werden und mit dem Register ADCON0 verknuepft werden */	
/*	 + ca. 50 Mikrosekunden (us) warten. Diese Zeit benoetigt der PIC um den internen 	*/
/*	   Kondensator mit der zu messenden Analogspannung zu laden.						*/
/*	 + Analog-Digital-Wandlung starten. Dazu das Flag GO (im Register ADCON0) setzen.   */
/*	 + Warten, bis der PIC mit der Wandlung fertig ist. Der PIC setzt das Flag GO auto- */
/*     matisch zurueck, wenn er mit der Analog-Digital-Wandlung fertig ist.				*/
/*   + Aus den Registern ADRESL und ADRESH das Ergebnis zusammensetzen und an das auf-  */
/*     rufende Unterprogramm (oder Hauptprogramm) zurueckgeben							*/
/*                                                                                      */
/* Anmerkung: 																			*/
/*	 Das Ausgabeformat der ADC-Wandlung, also wie die 10 Ergebnisbits in den Registern  */
/*   ADRESL und ADRESH abgelegt werden wird an anderer Stelle (z.B. im Unterprogramm    */
/*   INIT) konfiguriert.																*/
/****************************************************************************************/
void SOLL_ISTWERT_EINLESEN(void)
{
	// Istwert von ADC-Eingang AN0 (Kanal 0) einlesen und in der externen Variable 
	//   istwert sichern
	ADC(0);
	istwert = tmp_uebergabe_uns16;

	// Sollwert von ADC-Eingang AN2 (Kanal 2) einlesen und in der externen Variable 
	//   sollwert sichern
	ADC(1);
	sollwert = tmp_uebergabe_uns16;
}


/****************** Unterprogramm zum Reglerparameter einlesen **************************/

/****************************************************************************************/
/* PID_INIT:																			*/
/*																						*/
/* Aufgabe:																				*/
/*	 e_alt (Regelabweichung) und y_alt (Stellgroesse) mit 0 initialisieren				*/
/****************************************************************************************/

void PID_INIT(void)
{
	e_alt = 0;
	y_alt = 0;
}
 


/****************************************************************************************/
/* ADC:																					*/
/*																						*/
/* Aufgabe:																				*/
/*	 Analog-Digital-Wandlung mit dem PIC-interne ADC fuer den ausgewaehlten ADC-Kanal   */
/*	 starten, auf das Wandlungsergebnis warten und dieses dem aufrufenden Programm 		*/
/*	 zurueckgeben.																		*/
/*																						*/
/* Uebergabeparameter:                                                                  */
/*	 kanal: 			                                                          		*/
/*																						*/
/* Rueckgabeparameter:                                                                  */
/*	 ergebnis:  16-Bit-Ergebnis der Analog-Digital-Wandlung								*/
/*                                                                                      */
/* Vorgehensweise: 																		*/
/*	 + ADC-Kanal auswaehlen (Bits <3:5> im Register ADCON0). Dazu muss die Kanal-Nummer */
/*     an diese Position geschoben werden und mit dem Register ADCON0 verknuepft werden */	
/*	 + ca. 50 Mikrosekunden (us) warten. Diese Zeit benoetigt der PIC um den internen 	*/
/*	   Kondensator mit der zu messenden Analogspannung zu laden.						*/
/*	 + Analog-Digital-Wandlung starten. Dazu das Flag GO (im Register ADCON0) setzen.   */
/*	 + Warten, bis der PIC mit der Wandlung fertig ist. Der PIC setzt das Flag GO auto- */
/*     matisch zurueck, wenn er mit der Analog-Digital-Wandlung fertig ist.				*/
/*   + Aus den Registern ADRESL und ADRESH das Ergebnis zusammensetzen und an das auf-  */
/*     rufende Unterprogramm (oder Hauptprogramm) zurueckgeben							*/
/*                                                                                      */
/* Anmerkung: 																			*/
/*	 Das Ausgabeformat der ADC-Wandlung, also wie die 10 Ergebnisbits in den Registern  */
/*   ADRESL und ADRESH abgelegt werden wird an anderer Stelle (z.B. im Unterprogramm    */
/*   INIT) konfiguriert.																*/
/****************************************************************************************/
void REGLERPARAMETER_EINLESEN(void)
{
	// Kp von ADC-Eingang AN3 (Kanal 3) einlesen und in der externen Variable 
	//   Kp sichern
	ADC(2);
	Kp = tmp_uebergabe_uns16;

	// Ki von ADC-Eingang AN4 (Kanal 4) einlesen und in der externen Variable
	//   Ki sichern
	ADC(3);
	Ki = tmp_uebergabe_uns16;

	// Kd von ADC-Eingang AN5 (Kanal 5) einlesen und in der externen Variable
	//   Kd sichern
	ADC(4);
	Kd = tmp_uebergabe_uns16;
}



void PID_ROUTINE(void)
{
//	int16 abweichung;
//	int16 ausgabe;
	int16 temp;


 


	// Regelabweichung (Error) breechnen
	e_neu = (int16)(sollwert - istwert);


	if (pid_ein) 		
	{
		// PID-Regelung einschalten
		if (pid_p_aktiv)
		{
			// Proportionalanteil
			y_neu = e_neu * Kp;
		}
		else
		{
			y_neu = 0;
		}


		if (pid_i_aktiv)
		{
			// I-Anteil
			temp = e_alt + e_neu;
	//		temp = temp * (uns16)100;
	//		temp = temp / Tn;
	//		temp = temp / (uns16)2;
			temp = temp * Ki;
			
			y_neu = y_neu + y_alt;
			y_neu = y_neu + temp;
		}
		 

		if (pid_d_aktiv)
		{
			// D-Anteil
			temp = e_neu - e_alt;
	//		temp = temp * Kp;
	//		temp = temp / (uns16)100;		// Abtastzeit
	//		temp = temp * Tv;
			temp = temp * Kd;

			y_neu = y_neu + temp;
		}
	 
	}
	else
	{
		// PID-Regelung ausschalten
		
		y_neu = 0;
	}


	e_alt = e_neu;
	y_alt = y_neu;

	tmp_uebergabe_int16 = y_neu;
}


void AUSGABE_PID_PARAMETER(void)
{
	ausgabe_zaehler++;

	switch (ausgabe_zaehler)
	{
		case 1: 	
			// Sollwert am Display ausgeben
			GLCD_SETCURSOR_ZEILE(0);
			GLCD_SETCURSOR_SPALTE(0);
			GLCD_ZEICHEN_5x7('S');
			GLCD_ZEICHEN_5x7('o');
			GLCD_ZEICHEN_5x7(':');
 			AUSGABEHEX(sollwert.high8);
			GLCD_ZEICHEN_5x7('.');
			AUSGABEHEX(sollwert.low8);
			GLCD_ZEICHEN_5x7('h');
			break;

		case 2: 
			// Istwert am Display ausgeben
			GLCD_SETCURSOR_ZEILE(1);
			GLCD_SETCURSOR_SPALTE(0);
			GLCD_ZEICHEN_5x7('I');
			GLCD_ZEICHEN_5x7('s');
			GLCD_ZEICHEN_5x7(':');
 			AUSGABEHEX(istwert.high8);
			GLCD_ZEICHEN_5x7('.');
			AUSGABEHEX(istwert.low8);
			GLCD_ZEICHEN_5x7('h');
			break;

		case 3: 
			// Regelabweichung (Error) am Display ausgeben
			GLCD_SETCURSOR_ZEILE(2);
			GLCD_SETCURSOR_SPALTE(0);
			GLCD_ZEICHEN_5x7('e');
			GLCD_ZEICHEN_5x7(':');
			GLCD_ZEICHEN_5x7(' ');
			AUSGABEHEX(e_neu.high8);
			GLCD_ZEICHEN_5x7('.');
 			AUSGABEHEX(e_neu.low8);
			GLCD_ZEICHEN_5x7('h');
			break;

		case 4: 
			// PID-Regelung ein oder aus
			GLCD_SETCURSOR_ZEILE(3);
			GLCD_SETCURSOR_SPALTE(0);
			GLCD_ZEICHEN_5x7('P');
			GLCD_ZEICHEN_5x7('I');
			GLCD_ZEICHEN_5x7('D');
			GLCD_ZEICHEN_5x7(' ');
			if (pid_ein) 		
			{
				GLCD_ZEICHEN_5x7('E');
				GLCD_ZEICHEN_5x7('i');
				GLCD_ZEICHEN_5x7('n');
			}
			else
			{
				GLCD_ZEICHEN_5x7('A');
				GLCD_ZEICHEN_5x7('u');
				GLCD_ZEICHEN_5x7('s');
			}
			break;

		case 5: 
			// Kp am Display ausgeben
			GLCD_SETCURSOR_ZEILE(0);
			GLCD_SETCURSOR_SPALTE(60);
			GLCD_ZEICHEN_5x7('K');
			GLCD_ZEICHEN_5x7('p');
			GLCD_ZEICHEN_5x7(':');
			if (pid_p_aktiv)
			{
 				AUSGABEHEX(Kp.high8);
				GLCD_ZEICHEN_5x7('.');
 				AUSGABEHEX(Kp.low8);
				GLCD_ZEICHEN_5x7('h');
			}
			else
			{
				GLCD_ZEICHEN_5x7('a');
				GLCD_ZEICHEN_5x7('u');
				GLCD_ZEICHEN_5x7('s');
 			}
			break;

		case 6: 
			// Ki am Display ausgeben
			GLCD_SETCURSOR_ZEILE(1);
			GLCD_SETCURSOR_SPALTE(60);
			GLCD_ZEICHEN_5x7('K');
			GLCD_ZEICHEN_5x7('i');
			GLCD_ZEICHEN_5x7(':');
			if (pid_i_aktiv)
			{
 				AUSGABEHEX(Ki.high8);
				GLCD_ZEICHEN_5x7('.');
 				AUSGABEHEX(Ki.low8);
				GLCD_ZEICHEN_5x7('h');
			}
			else
			{
				GLCD_ZEICHEN_5x7('a');
				GLCD_ZEICHEN_5x7('u');
				GLCD_ZEICHEN_5x7('s');
 			}
			break;

		case 7: 
			// Kd am Display ausgeben
			GLCD_SETCURSOR_ZEILE(2);
			GLCD_SETCURSOR_SPALTE(60);
			GLCD_ZEICHEN_5x7('K');
			GLCD_ZEICHEN_5x7('d');
			GLCD_ZEICHEN_5x7(':');
			if (pid_d_aktiv)
			{
 				AUSGABEHEX(Kd.high8);
				GLCD_ZEICHEN_5x7('.');
 				AUSGABEHEX(Kd.low8);
				GLCD_ZEICHEN_5x7('h');
			}
			else
			{
				GLCD_ZEICHEN_5x7('a');
				GLCD_ZEICHEN_5x7('u');
				GLCD_ZEICHEN_5x7('s');
 			}
			break;

		case 8: 
			// Ausgangswert am Display ausgeben
			GLCD_SETCURSOR_ZEILE(3);
			GLCD_SETCURSOR_SPALTE(60);
			GLCD_ZEICHEN_5x7('a');
			GLCD_ZEICHEN_5x7('u');
			GLCD_ZEICHEN_5x7(':');
 			AUSGABEHEX(y_neu.high8);
			GLCD_ZEICHEN_5x7('.');
 			AUSGABEHEX(y_neu.low8);
			GLCD_ZEICHEN_5x7('h');

			ausgabe_zaehler = 0;
			break;
	}

}
