/****************************************************************************************/
/* Header zum Demoprojekt 2 zur Ansteuerung des Grafik-LCD (mit SED1520-Controller) 	*/
/* in C (CC5X)																			*/
/*																						*/
/* Entwickler: Buchgeher Stefan															*/
/* Entwicklungsbeginn der Software: 5. Maerz 2006										*/
/* Funktionsfaehig seit: 2. April 2006 					 							    */
/* Letzte Bearbeitung: 1. Mai 2006													*/
/****************************************************************************************/

#ifndef  __PROJEKT   
#define  __PROJEKT    


/****************** Pragma-Anweisungen **************************************************/
#pragma chip PIC16F877				// PICmicro Device
//#pragma library 1					//.. library functions that are deleted if unused


/****************** Externe Strukturen  *************************************************/
/* keine externen Strukturen verwendet */


/****************** Externe Register  ***************************************************/
// Externe Uebergaberegister (zwischen den Unterprogrammen)
extern bank0 uns16 tmp_uebergabe_uns16;
extern bank0 int16 tmp_uebergabe_int16;

extern bank0 char ausgabe_zaehler;

// Externe Register zur Ansteuerung des Grafik-LC-Display
extern bank0 unsigned char hlp_zeilen_zaehler;
extern bank0 unsigned char hlp_spalten_zaehler;

// Externe Register zur PID-Regelung
extern bank0 uns16 sollwert;		// Sollwert
extern bank0 uns16 istwert;			// Istwert

extern bank0 int16 e_alt;			// vorhergehende Regelabweichung
extern bank0 int16 e_neu;			// neue Regelabweichung
extern bank0 int16 y_alt;			// vorhergehende Stellgroesse
extern bank0 int16 y_neu;			// neue Stellgroesse


extern bank0 uns16 Kp;				// Reglerparameter P-Anteil			
extern bank0 uns16 Ki;				// Reglerparameter I-Anteil	
extern bank0 uns16 Kd;				// Reglerparameter D-Anteil	



extern bank0 unsigned char PID_STATUS;	// Statusregister fuer PID-Regler
bit pid_ein @ PID_STATUS.0;			// Globale aktivierung der PID-Regelung
bit pid_p_aktiv @ PID_STATUS.5;		// Bit gesetzt: P-Anteil wird im PID-Algorithmus 
									//   beruecksichtigt
bit pid_i_aktiv @ PID_STATUS.6;		// Bit gesetzt: I-Anteil wird im PID-Algorithmus 
									//   beruecksichtigt
bit pid_d_aktiv @ PID_STATUS.7;		// Bit gesetzt: D-Anteil wird im PID-Algorithmus 
									//   beruecksichtigt


/****************** Bits in den externen Registern **************************************/
/* keine externen Register verwendet */


/****************** Konstanten **********************************************************/
/* keine Konstanten */


/****************** Zeichensaetzen einbinden  *******************************************/
#include "zeichensatz5x7_a.inc"		// Zeichensatz (5x7) einbinden
#include "zeichensatz8x14_a.inc"	// Zeichensatz (8x14) einbinden
#include "zeichensatz10x22_b.inc"	// Zeichensatz (10x22) einbinden


/****************** Portbelegung ********************************************************/
/* Port B */
#pragma char GLCD_CTRL			@ PORTB       
#pragma char GLCD_CTRL_TRIS 	@ TRISB 

#pragma bit GLCD_WR				@ PORTB.1
#pragma bit GLCD_RD				@ PORTB.2
#pragma bit GLCD_CS1			@ PORTB.3
#pragma bit GLCD_CS2			@ PORTB.4
#pragma bit GLCD_A0				@ PORTB.5

/* Port D */
#pragma char GLCD_DATA 			@ PORTD  
#pragma char GLCD_DATA_TRIS 	@ TRISD 


/****************** Funktionsprototypen *************************************************/
/* Initialisierung des Mikrocontroller */
extern page0 void INIT(void);

/* Unterprogramme zur Datenausgabe */
//extern page0 void AUSGABE(void);
extern page0 void AUSGABEBCD(unsigned char wert);
extern page0 void AUSGABEHEX(unsigned char wert);

//void AUSGABEBCD_10x22(unsigned char wert); 

/* Unterprogramm zur Analog-Digital-Wandlung */
extern page0 void ADC(char kanal);

/* Unterprogramm zur Ist- und Sollwerteingabe */
extern page0 void Soll_Istwerte_einlesen(void);

/* Unterprogramm zur Motoransteuerung mittels IC TLE5205-2 und PWM */
extern page0 void PWM_TLE5205(void);

/* Sonstige Unterprogramme */
extern page0 void delay_us(char mikro);

/* Unterprogramme fuer Grafik-LC-Display */
extern page0 void GLCD_INIT(void);	

extern page0 void GLCD_COMMAND(bank0 unsigned char command);
extern page0 void GLCD_DATA1(bank0 unsigned char data);
extern page0 void GLCD_DATA2(bank0 unsigned char data);
extern page0 void GLCD_DATA_ZEICHEN(bank0 unsigned char data);

extern page0 void GLCD_CLEAR(void);

extern page0 void GLCD_SETCURSOR_ZEILE(bank0 unsigned char zeile);
extern page0 void GLCD_SETCURSOR_SPALTE(bank0 unsigned char spalte);

extern page0 void GLCD_ZEICHEN_5x7(bank0 unsigned char zeichen);
//	extern page0 void GLCD_STRING_5x7(const char *pString);
extern page0 void GLCD_ZEICHEN_8x14(bank0 unsigned char zeichen);
extern page0 void GLCD_ZEICHEN_10x22(bank0 unsigned char zeichen);


/* Unterprogramm zur PID-Regelung */
extern page1 void SOLL_ISTWERT_EINLESEN(void);
extern page1 void REGLERPARAMETER_EINLESEN(void);

extern page1 void PID_INIT(void);
extern page1 void PID_ROUTINE(void);

extern page1 void AUSGABE_PID_PARAMETER(void);


#endif 
