/****************************************************************************************/
/* Demonstrationsbeispiel eines digitalen PID-Regler am Beispiel eines inversen Pendels */
/* in C (CC5X)																			*/
/*																						*/
/* Entwickler: Buchgeher Stefan															*/
/* Entwicklungsbeginn der Software: 16. Maerz 2006										*/
/* Funktionsfaehig seit:  								 							    */
/* Letzte Bearbeitung: 27. April 2006													*/
/****************************************************************************************/

 
/****************** Include-Dateien *****************************************************/
//#include "16f877.h"
#include "PROJEKT.H"
#include <int16CXX.H>	 			// Ist fuer die Interrupts notwendig
#include <INLINE.H> 				// Ist fuer Assembleranweisungen notwendig


/****************** Pragma-Anweisungen **************************************************/
//#pragma chip PIC16F877				// PICmicro Device
//#pragma library 1					//.. library functions that are deleted if unused


/****************** Strukturen  *********************************************************/
/* keine Strukturen verwendet */


/****************** Externe Register  ***************************************************/
char FLAGISRHP;   					// beinhaltet Botschaftsflags ISR -> HP
char ZAEHLERISR8MSEK;				// Zaehlregister fuer 10ms-Zeitbasis

//uns16 tmp_uebergabe_uns16;
char ausgabe_zaehler;


/****************** Bits in den externen Registern **************************************/
/* Register FLAGSISRHP */
bit	FLAG8MSEK					@ FLAGISRHP.0;
 

/****************** Portbelegung ********************************************************/
/* Port B */
/*#pragma char GLCD_CTRL			@ PORTB       
#pragma char GLCD_CTRL_TRIS 	@ TRISB 

bit GLCD_WR						@ GLCD_CTRL.1;
bit	GLCD_RD						@ GLCD_CTRL.2;
bit GLCD_CS1					@ GLCD_CTRL.3;
bit GLCD_CS2					@ GLCD_CTRL.4;
bit GLCD_A0						@ GLCD_CTRL.5;*/

/* Port D */
/*#pragma char GLCD_DATA 			@ PORTD       
#pragma char GLCD_DATA_TRIS 	@ TRISD */


/****************** Konstanten **********************************************************/
/* Konstanten fuer Zeitbasen */
//#define 	KONSTISR8MSEK 		9  // fuer 8 ms
#define 	KONSTISR8MSEK 		112  // fuer 100 ms



/****************** Konfigurations-Bits *************************************************/ 
#pragma config |= 0b.11110100111010
/*         		     ++------++------- Bit 13-12, 5-4 (CP1:CP0): FLASH Memory Code Protection
					   ||||||  ||||        0  0  : 0000h to 1FFFh code protected
					   ||||||  ||||        0  1  : 1000h to 1FFFh code protected
					   ||||||  ||||        1  0  : 1F00h to 1FFFh code protected
					   ||||||  ||||    ->  1  1  : Code protection off
					   +-------------- Bit 11 (DEBUG): In-Circuit Debugger Mode
						|||||  ||||         0 : DEBUG on  (= enabled)
						|||||  ||||    ->   1 : DEBUG off (= disabled)
						+------------- Bit 10 : Reserve
						 +------------ Bit  9 (WRT): FLASH Programm Memory Write Enable
						  |||  ||||    ->   0 : WRT off   (= diasbled)
						  |||  ||||         1 : WRT on    (= enabled)
						  +----------- Bit  8 (CPD): Data EE Memory Code Protection
						   ||  ||||         0 : CPD on    (= enabled)
						   ||  ||||    ->   1 : CPD off   (= disabled)
						   +---------- Bit  7 (LVP): Low Voltage ICSP Enable Bit
							|  ||||    ->   0 : LVP off   (= disabled)
							|  ||||         1 : LVP on    (= enabled)
							+--------- Bit  6 (BODEN): Brown-Out Detection
							   ||||    ->   0 : BODEN on  (= disabled)
							   ||||         1 : BODEN off (= enabled)
							   +------ Bit  3 (PWRTE): Power Up Timer
								|||         0 : PWRT on   (= enabled)
								|||    ->   1 : PWRT off  (= disabled) 
								+----- Bit  2 (WDTE): Watchdog Timer
								 ||    ->   0 : WDT off   (= disabled)
								 ||         1 : WDT on    (= enabled))
								 ++--- Bit 1-0 (FOSC2:FOSC0): Oszillator Selection
									      0 0 : LP Oszillator
										  0 1 : XT Oszillator
							 	       -> 1 0 : HS Oszillator
 										  1 1 : RC Oszillator
*/


/****************** Funktionsprototypen *************************************************/
/* Initialisierung des Mikrocontroller */
//void INIT(void);

/* Unterprogramme zur Datenausgabe */
//void AUSGABE(void);
//void AUSGABEBCD(unsigned char wert);
//void AUSGABEBCD_10x22(unsigned char wert); 



/* Unterprogramm zur Analog-Digital-Wandlung */
//uns16 ADC(char kanal);

/* Unterprogramm zur Ist- und Sollwerteingabe */
//void Soll_Istwerte_einlesen(void);


/* Unterprogramm zur Motoransteuerung mittels IC TLE5205-2 und PWM */
//void PWM_TLE5205(uns16 wert);


/* Sonstige Unterprogramme */
//void delay_us(char mikro);




/****************** ISR - Timer0 ********************************************************/

/****************************************************************************************/
/* Interrupt Service Routine:								     						*/
/* 											     										*/
/* Aufruf:										     									*/
/*  	alle 8 ms									     								*/
/* 											     										*/
/* Aufgaben:										     								*/
/* 		+ w-Register (=Arbeitsregister) und Status-Register zwischenspeichern 			*/
/*		  (int_save_registers)															*/
/*		+ Zeitbasis fuer 8 Millisekunde erzeugen						   				*/
/* 		+ Das Timer-Interrupt-Flag T0IF wieder loeschen					     			*/
/*		+ w-Register (=Arbeitsregister) und Statusregister wiederherstellen 	     	*/
/*		  (int_restore_registers)														*/
/****************************************************************************************/
#pragma origin 4						 

interrupt InterruptRutine(void)			// Interruptroutine
{
	int_save_registers					// W, STATUS (und  PCLATH) retten
 
	// Beginn der eigentlichen ISR-Routine
	ZAEHLERISR8MSEK--;
	if (ZAEHLERISR8MSEK == 0)
	{	
		FLAG8MSEK = 1;					// Botschaftsflag setzen
		ZAEHLERISR8MSEK = KONSTISR8MSEK;  // Zaehlregister fuer den 8-ms-Takt mit
	}									// der Konstante KONSTISR8MSEK neu laden
							
	T0IF = 0; 							// Timer-Interrupt-Flag T0IF wieder loeschen	
	// Ende der eigentlichen ISR-Routine

	int_restore_registers				// W, STATUS (und PCLATH) Wiederherstellen
}

 


/****************** Unterprogramme und Funktionen ***************************************/

/****************** externe Unterprogramme einbinden ************************************/
/* Mathematik-Routinen */
#include <MATH16.H>

 

/****************************************************************************************/
/* INIT:																				*/
/*																						*/
/* Aufgabe:																				*/
/*	 Initialisierung des Prozessor:														*/
/*		+ Timer 0 (TMR0) loeschen														*/
/*		+ Timer 0-ISR soll alle 8ms aufgerufen werden, daher den Vorteiler mit 16 laden */
/*		  (bei einem 18,432MHz-Quarz													*/
/*   	+ Port A als Eingang (fuer ADC), Port B und Port D als Ausgang definieren,		*/
/*      + ADC konfigurieren																*/
/*																						*/
/* Uebergabeparameter:                                                                  */
/*	 zeichen			                                                                */
/*																						*/
/* Rueckgabeparameter:                                                                  */
/* 	 keiner																				*/
/****************************************************************************************/
void INIT(void)
{
	// Timer-0-Interrupt
	TMR0 = 0;							// Timer 0 auf 0 voreinstellen
	OPTION = 0b.1000.0011;				// Pull-Up-Widerstaende am Port B deaktivieren
										// Timer-ISR alle 8ms aufrufen (Vorteiler = 16)

	// Ports
	TRISA = 0xFF;						// Port A  und Port E muessen bei verwendung als 
	TRISE = 0xFF;						//   ADC als Eingang konfiguriert werden
 	TRISB = 0;							// Port B als Ausgang definieren
	TRISC = 0b.1111.1001;				// Port C: Bits 1 und 2 als Ausgang definieren
										// Port C: Bits 0 und 3 bis 7 als Eingang definieren	
	TRISD = 0;							// Port D als Ausgang definieren

	// ADC
	ADON = 1;							// ADC global einschalten
	ADCON1 = 0b.1000.0010;				// Ganzer Port A als Analog; Vref+ = Vdd, Vref- = Vss 
										//   und linksbuendig
	ADCS0 = 0;							// Geschwindigkeit des ADC mit ADCS0 und ADCS1
	ADCS1 = 1;
									 
	ZAEHLERISR8MSEK = KONSTISR8MSEK;	// Zaehlregister fuer 8-ms-Zeitbasis
										// initialisieren

	// PWM-Module
	T2CON = 0b.0000.0100;				// Vorteiler Timer 2: 1:1, Timer 2 einschalten
	PR2 = 0b.1111.1111;					// 
	CCP1CON = 0b.0000.1100;				// PWM 1, Ausgang RC2
	CCP2CON = 0b.0000.1100;				// PWM 2, Ausgang RC1
	CCPR1L = 255;						// IC TLE5205 bremst (CCP1 (RC2) <-> IN1
	CCPR2L = 0;							// IC TLE5205 bremst (CCP2 (RC1) <-> IN2


	ausgabe_zaehler = 0;
}
									 

 

/****************************************************************************************/
/* AUSGABEHEX:																			*/
/*																						*/
/* Aufgabe:																				*/
/*	 Zeichen und Texte (Strings) an beliebiger Position am Grafik-LC-Display ausgeben.	*/
/*																						*/
/* Uebergabeparameter:                                                                  */
/*	 keine			                                                          		    */
/*																						*/
/* Rueckgabeparameter:                                                                  */
/*	 keiner																				*/
/*                                                                                      */
/* Vorgehensweise: 																		*/
/*	 + Grafik-LC-Display loeschen (Unterprogramm GLCD_CLEAR)							*/
/*	 + Cursor an eine beliebige Stelle positionieren (Unterprogramm GLCD_SETCURSOR)		*/
/*	 + Einzelne Zeichen (mit dem Unterprogramm GLCD_ZEICHEN) oder ganze Texte (mit dem  */
/*	   Unterprogramm GLCD_STRING) am Grafik-LC-Display ausgeben.						*/
/****************************************************************************************/
void AUSGABEHEX(unsigned char wert)
{
	unsigned char help;
	unsigned char help_modulo;

	// Erste Stelle

	help = wert;
	SWAPF(help,1);
	help = help & 0x0F;
	GLCD_ZEICHEN_5x7(help);	

		
	// Zweite Stelle
	wert = wert & 0x0F;
	GLCD_ZEICHEN_5x7(wert);	
}


/****************************************************************************************/
/* AUSGABEBCD:																			*/
/*																						*/
/* Aufgabe:																				*/
/*	 Zeichen und Texte (Strings) an beliebiger Position am Grafik-LC-Display ausgeben.	*/
/*																						*/
/* Uebergabeparameter:                                                                  */
/*	 keine			                                                          		    */
/*																						*/
/* Rueckgabeparameter:                                                                  */
/*	 keiner																				*/
/*                                                                                      */
/* Vorgehensweise: 																		*/
/*	 + Grafik-LC-Display loeschen (Unterprogramm GLCD_CLEAR)							*/
/*	 + Cursor an eine beliebige Stelle positionieren (Unterprogramm GLCD_SETCURSOR)		*/
/*	 + Einzelne Zeichen (mit dem Unterprogramm GLCD_ZEICHEN) oder ganze Texte (mit dem  */
/*	   Unterprogramm GLCD_STRING) am Grafik-LC-Display ausgeben.						*/
/****************************************************************************************/
void AUSGABEBCD(unsigned char wert)
{
	unsigned char help;
	unsigned char help_modulo;

	// Hunderterstelle
	help = (char)(wert / 100);
	GLCD_ZEICHEN_5x7(help);	
   	help_modulo = wert % 100;

	// Zehnerstelle
	help = (char)(help_modulo / 10);
	GLCD_ZEICHEN_5x7(help);	
   	help_modulo = help_modulo % 10;

	// Einerstelle
	GLCD_ZEICHEN_5x7(help_modulo);	
}


//void AUSGABEBCD_10x22(unsigned char wert)
//{
//	unsigned char help;
//	unsigned char help_modulo;

	// Hunderterstelle
//	help = (char)(wert / 100);
//	GLCD_ZEICHEN_10x22(help);	
//   help_modulo = wert % 100;

	// Zehnerstelle
//	help = (char)(help_modulo / 10);
//	GLCD_ZEICHEN_10x22(help);	
//  help_modulo = help_modulo % 10;

	// Einerstelle
//	GLCD_ZEICHEN_10x22(help_modulo);	
//}



void delay_us(char mikro)
{
	#asm
	wdh9	nop
	nop
	decfsz	mikro,f
	goto	wdh9
	#endasm
}



/****************** Unterprogramm zur Analog-Digital-Wandlung ***************************/

/****************************************************************************************/
/* ADC:																					*/
/*																						*/
/* Aufgabe:																				*/
/*	 Analog-Digital-Wandlung mit dem PIC-interne ADC fuer den ausgewaehlten ADC-Kanal   */
/*	 starten, auf das Wandlungsergebnis warten und dieses dem aufrufenden Programm 		*/
/*	 zurueckgeben.																		*/
/*																						*/
/* Uebergabeparameter:                                                                  */
/*	 kanal: 			                                                          		*/
/*																						*/
/* Rueckgabeparameter:                                                                  */
/*	 6-Bit-Ergebnis der Analog-Digital-Wandlung	im EXTERNEN UEBERGABEREGISTER			*/
/*   (tmp_uebergabe_uns16)																*/
/*                                                                                      */
/* Vorgehensweise: 																		*/
/*	 + ADC-Kanal auswaehlen (Bits <3:5> im Register ADCON0). Dazu muss die Kanal-Nummer */
/*     an diese Position geschoben werden und mit dem Register ADCON0 verknuepft werden */	
/*	 + ca. 50 Mikrosekunden (us) warten. Diese Zeit benoetigt der PIC um den internen 	*/
/*	   Kondensator mit der zu messenden Analogspannung zu laden.						*/
/*	 + Analog-Digital-Wandlung starten. Dazu das Flag GO (im Register ADCON0) setzen.   */
/*	 + Warten, bis der PIC mit der Wandlung fertig ist. Der PIC setzt das Flag GO auto- */
/*     matisch zurueck, wenn er mit der Analog-Digital-Wandlung fertig ist.				*/
/*   + Aus den Registern ADRESL und ADRESH das Ergebnis zusammensetzen und an das auf-  */
/*     rufende Unterprogramm (oder Hauptprogramm) zurueckgeben							*/
/*                                                                                      */
/* Anmerkung: 																			*/
/*	 Das Ausgabeformat der ADC-Wandlung, also wie die 10 Ergebnisbits in den Registern  */
/*   ADRESL und ADRESH abgelegt werden wird an anderer Stelle (z.B. im Unterprogramm    */
/*   INIT) konfiguriert.																*/
/****************************************************************************************/
void ADC(char kanal)
{
//	uns16	ergebnis;
	

	ADCON0 = ADCON0 & 0b.1100.0111;
	kanal = kanal << 3;
	ADCON0 = ADCON0 | kanal;
		
	delay_us(50);

	GO = 1;
	while(GO);

	tmp_uebergabe_uns16.low8 = ADRESL;			// externes Uebergaberegister
	tmp_uebergabe_uns16.high8 = ADRESH;

//	return ergebnis;
}


/****************** Unterprogramm zur Motoransteuerung mittels IC TLE5205-2 und PWM *****/

/****************************************************************************************/
/* PWM_TLE5205:																			*/
/*																						*/
/* Aufgabe:																				*/
/*	 Analog-Digital-Wandlung mit dem PIC-interne ADC fuer den ausgewaehlten ADC-Kanal   */
/*	 starten, auf das Wandlungsergebnis warten und dieses dem aufrufenden Programm 		*/
/*	 zurueckgeben.																		*/
/*																						*/
/* Uebergabeparameter:                                                                  */
/*	 kanal: 			                                                          		*/
/*																						*/
/* Rueckgabeparameter:                                                                  */
/*	 ergebnis:  16-Bit-Ergebnis der Analog-Digital-Wandlung								*/
/*                                                                                      */
/* Vorgehensweise: 																		*/
/*	 + ADC-Kanal auswaehlen (Bits <3:5> im Register ADCON0). Dazu muss die Kanal-Nummer */
/*     an diese Position geschoben werden und mit dem Register ADCON0 verknuepft werden */	
/*	 + ca. 50 Mikrosekunden (us) warten. Diese Zeit benoetigt der PIC um den internen 	*/
/*	   Kondensator mit der zu messenden Analogspannung zu laden.						*/
/*	 + Analog-Digital-Wandlung starten. Dazu das Flag GO (im Register ADCON0) setzen.   */
/*	 + Warten, bis der PIC mit der Wandlung fertig ist. Der PIC setzt das Flag GO auto- */
/*     matisch zurueck, wenn er mit der Analog-Digital-Wandlung fertig ist.				*/
/*   + Aus den Registern ADRESL und ADRESH das Ergebnis zusammensetzen und an das auf-  */
/*     rufende Unterprogramm (oder Hauptprogramm) zurueckgeben							*/
/*                                                                                      */
/* Anmerkung: 																			*/
/*	 Das Ausgabeformat der ADC-Wandlung, also wie die 10 Ergebnisbits in den Registern  */
/*   ADRESL und ADRESH abgelegt werden wird an anderer Stelle (z.B. im Unterprogramm    */
/*   INIT) konfiguriert.																*/
/****************************************************************************************/
void PWM_TLE5205(void)
{
	int16 wert;									// 16 Bit signed


	wert = tmp_uebergabe_int16;					// 16-Bit-Uebergabewert 
												//   (tmp_uebergabe_int16) im Register
												//   Wert sichern		
	if (wert.15 == 0)
	{
		// wert ist eine negative Zahl
		wert = wert << 1;		

		COMF(wert.high8,1);						// Bits invertiern	
			
		Carry = 0;								// Carry-Flag loeschen
		wert.high8 = wert.high8 >> 2;			// Ausgleich des Totgangs des Motors

		CCPR1L = wert.high8;
		CCPR2L = wert.high8;;	
	}
	else
	{
		// wert ist eine positive Zahl
		wert = wert << 1;

		Carry = 0;								// Carry-Flag loeschen
		wert.high8 = wert.high8 >> 2;			// Ausgleich des Totgangs des Motors


		CCPR2L = wert.high8;
		CCPR1L = 255;
	}
	
}







/****************** Hauptprogramm *******************************************************/

/****************************************************************************************/
/* Aufgaben des Hauptprogramms:							     							*/
/*	 + Controller initialisieren (Unterprogramm INIT)				     				*/
/*   + Grafik-LC-Display initialisieren (Unterprogramm GLCD_INIT)						*/
/*   + Grafik-LC-Display loeschen (Unterprogramm GLCD_CLEAR)							*/
/*	 + Interrupt freigeben								     							*/
/*	 + Taetigkeiten/Unterprogramme, die alle 10 Sekunden durchgefuehrt werden muessen:	*/
/*		+ blabl			*/
/****************************************************************************************/
void main(void)
{
	uns16 temp;
 

	INIT();								// Controller initialisieren
    GLCD_INIT();						// Grafik-LC-Display initialisieren
 
	GLCD_CLEAR();						// Grafik-LC-Display loeschen

	PID_INIT();							// Register fuer PID-Regelung initialisieren

	INTCON = 0b.1010.0000;				// Timer0 freigeben durch Setzen von
										// GIE und T0IE im Register INTCON
	while(1)
	{
		if (FLAG8MSEK)					// Alle 8 Millisekunden
		{
			// aktuelle Soll- und Istwerte einlesen
			SOLL_ISTWERT_EINLESEN();

			// Reglerparameter einlesen
			REGLERPARAMETER_EINLESEN();

			// Regelalgorithmus
			PID_ROUTINE();

			// Stellgroesse ausgeben
			PWM_TLE5205();

			// Reglerparameter und Ausgabewert am Display ausgeben
			AUSGABE_PID_PARAMETER();
			
			// Jumper einelesen und im Register PID_STATUS sichern
			PID_STATUS = PORTC;
			
 
			FLAG8MSEK = 0;				// Botschaftsflag fuer den 10-Milli-
		}								// Sekunden-Takt loeschen
		
	}	
} 
